/*  "Expressive" compiler
 *
 *  Program that tests the code generator
 *  Philippe Altherr, Nikolay Mihaylov
 *
 *  $Id: GeneratorTest.java,v 1.3 2003/06/04 13:36:16 mihaylov Exp $
 */

package expressive;

import java.io.InputStream;
import java.io.PrintWriter;
import java.io.PrintStream;

/**
 * This class tests the interpreter.
 *
 * usage: java expressive.GeneratorTest <options> [source_file [output-file]]
 * with the following options:
 *   -debug    Print the debug messages
 *   -run      Run the generated program if there were no errors
 *   -trace    Trace the execution of the program
 *   -? -help  Show this help
 */
public class GeneratorTest extends AbstractMain implements Tokens {

    //########################################################################
    // Instance variables

    /** Shows if the generated code should be executed. */
    public boolean run = false;

    /** Shows if the executed should be traced. */
    public boolean trace = false;

    //########################################################################
    // Instance methods

    /** Handle command line options and return
     *  the number of the consumed arguments. */
    public int handleOption(String[] args, int i) {
	if (args[i].equals("-run")) {
	    run = true;
	    return 1;
	} else if (args[i].equals("-trace")) {
	    trace = true;
	    return 1;
	} else
	    return super.handleOption(args, i);
    }

    /** Print help info. */
    public void printHelp(PrintStream out) {
        printUsage(out);
        out.println("with the following options :");
        out.println("  -debug    Print the debug messages");
        out.println("  -run      Run the generated program if there were no errors");
        out.println("  -trace    Trace the execution of the program");
        out.println("  -? -help  Show this help");
    }

    /** Main program. */
    public void run(Global global, InputStream in, PrintWriter out) {
	Scanner scanner = new Scanner(in);
	Parser parser = new Parser(scanner);
	try {
	    Tree prog = (Tree)parser.parse().value;
	    (new Analyzer(global)).analyze(prog);
	    if (global.errors() == 0) {
		Generator gen = new Generator(global, out);
		gen.generate(prog);
		if (run) {
		    gen.getVM().run(out, trace);
		} else {
		    gen.getVM().printCode(out);
		}
	    }
	} catch (Exception e) { e .printStackTrace(); }
    }

    //########################################################################
    // Main method

    public static void main(String[] args) {
        new GeneratorTest().run(args);
    }

    //########################################################################
}
