/*  "Expressive" compiler
 *
 *  Symbol representation.
 *  Nikolay Mihaylov
 *
 *  $Id: Symbol.java,v 1.2 2003/03/25 12:12:04 mihaylov Exp $
 */

package expressive;

/**
 * This class implements symbols that represent identifiers.
 */
public class Symbol {

    //########################################################################
    // Instance variables

    /** Name of the symbol. */
    public final String name;

    /** Type of the symbol. */
    public final Type type;

    /** Next symbol in the symbol table. */
    public final Symbol next;

    /** Offset of the symbol in memory.
	Assigned during code generation. */
    private int offset = -1;

    /** Value of the variable denoted by the symbol.
     *  Used for interpretation. */
    public int value;

    //########################################################################
    // Constructor

    public Symbol(String name, Type type, Symbol next) {
	this.name = name;
	this.type = type;
	this.next = next;
    }

    //########################################################################
    // Methods

    /** Set the offset of a symbol. Abort execution
     *  if offset has already been set. */
    public int setOffset(int offset) {
	if (this.offset >= 0)
	    Global.instance.fatal(Position.UNDEFINED,
				  "Forbidden to change symbol's offset: " +
				  this);
	this.offset = offset;
	return offset;
    }

    /** Get the offset of the symbol. */
    public int getOffset() {
	return offset;
    }

    /** Returns a string representation of the symbol. */
    public String toString() {
	return name + ": " + type;
    }

    //########################################################################

}
