/*  "Expressive" compiler
 *
 *  Type representation.
 *  Nikolay Mihaylov
 *
 *  $Id: Type.java,v 1.3 2003/03/25 12:43:06 mihaylov Exp $
 */

package expressive;

/**
 * This class represents the types of tree nodes, expressions and symbols.
 */
public final class Type {

    //########################################################################
    // Static Type instances

    /** Represents the absence of type. */
    public static final Type NoType = new Type();

    /** Represents the impossibility to assign a type. */
    public static final Type Bad = new Type();

    /** Represents Integer type. */
    public static final Type Int = new Type();

    /** Represents Boolean type. */
    public static final Type Bool = new Type();

    /** Represents String type. */
    public static final Type String = new Type();

    //########################################################################
    // Constructors

    /** Private constructor. Prevents creating new instances. */
    private Type() {}

    //########################################################################
    // Methods

    /** Returns true if the type can be the type of valid expression. */
    public boolean isExprType() {
	return (this == Int) || (this == Bool);
    }

    /** Returns string representation of a type. */
    public String toString() {
	if (this == Int) return "Int";
	else if (this == Bool) return "Bool";
	else if (this == String) return "String";
	else if (this == Bad) return "<Bad>";
	else if (this == NoType) return "<No type>";
	else return "<UNKNOWN TYPE>";
    }

    //########################################################################

}
