/*
 * $Id: Instructions.java,v 1.1 2003/03/28 11:42:08 mihaylov Exp $
 */

package expressive.evm;

public interface Instructions {
    // The order of the instructions are important.

    //########################################################################
    // Instructions without arguments.

    /** Halts the execution. */
    int BREAK = 0;

    /** Removes the top of the stack:  [bottom, x] -> [bottom] */
    int POP = 1;

    /** Duplicates the top of the stack:  [bottom, x] -> [bottom, x, x] */
    int DUP = 2;

    /** [bottom, x, y] -> [bottom, x + y] */
    int ADD = 3;

    /** [bottom, x, y] -> [bottom, x - y] */
    int SUB = 4;

    /** [bottom, x, y] -> [bottom, x * y] */
    int MUL =5;

    /** [bottom, x, y] -> [bottom, x / y] */
    int DIV = 6;

    /** [bottom, x, y] -> [bottom, x % y] */
    int REM = 7;

    /** [bottom, x, y] -> [bottom, x ^ y] */
    int POW = 8;

    /** [bottom, x, y] -> [bottom, if x < y then 1 else 0] */
    int CMPL = 9;

    /** [bottom, x, y] -> [bottom, if x <= y then 1 else 0] */
    int CMPLE = 10;

    /** [bottom, x, y] -> [bottom, if x == y then 1 else 0] */
    int CMPE = 11;

    /** [bottom, x] -> [bottom], printint x */
    int PRINTI = 12;

    /** [bottom, x] -> [bottom], printbool x */
    int PRINTB = 13;

    /** [bottom] -> [bottom, x] where x is an int suplied by the user. */
    int READI = 14;

    /** [bottom] -> [bottom, x] where x is a boolean suplied by the user. */
    int READB = 15;

    //########################################################################
    // Instructions with integer argument.

    /** Denotes instructions that have integer parameter. */
    int have_arg = 16;

    /** Pushes the integer x onto the stack: [bottom] -> [bottom, x] */
    int PUSH = 16;

    //########################################################################
    // Instructions that have a memop as argument

    /** Denotes instructions that have memory location as parameter. */
    int have_mem_arg=17;

    /** [bottom, x] -> [bottom], data(m) = x */
    int STORE = 17;
    /** x = data(m),  [bottom] -> [bottom, x] */
    int LOAD = 18;

    //########################################################################
    // Instructions with label as argument

    /** Denotes instructions that have label as paramater. */
    int have_label_arg=19;

    /** PC = L */
    int GOTO = 19;

    /** [bottom, x] -> [bottom], if x == 0 then PC = L */
    int JZ = 20;

    /** [bottom, x] -> [bottom], if x != 0 then PC = L */
    int JNZ = 21;
    
    //########################################################################
    // Instructions with string argument.

    /** Denotes instructions that have index in the string pool as parameter. */
    int have_string_arg = 22;

    /** [bottom] -> [bottom], printstring strings(s) */
    int PRINTS = 22;
 
    //########################################################################
    // 

    /** Denotes the last instruction. */
    int lastinstr = 22;
}
