/*  EVM Assembler
 *
 *  Scanner specification
 *  Erik Stenman
 *
 *  $Id: easm.lex,v 1.2 2003/06/03 09:04:57 mihaylov Exp $
 */

package expressive.evm.asm;

%%
%class AsmScanner
%implements AsmInstructions
%function next_token
%type Token

%eofval{
  return mkToken(EOF);
%eofval}

%char
%line

%state YYSTRING

%{
private int pos() {
  return expressive.Position.encode(yyline + 1, yychar);
}

private void newLine() {
    yychar = 0;
}

private Token mkToken(int tok) {
    return new Token(tok, pos()); 
}

private Token mkToken(int tok, int ivalue) {
    return new Token(tok, pos(), ivalue);
}

private Token mkToken(int tok, String svalue) {
    return new Token(tok, pos(), svalue);
}

private StringBuffer str;
private int strPos;

%}

letter=[a-zA-Z_]
digit=[0-9]
whitespace=[\t ]

%%
\n { newLine(); }

<YYINITIAL>{whitespace}              {/* Skip whitespace */}
<YYINITIAL>"/*"([^*]|"*"[^/])*"*/"   {/* Skip multi-line comments */}
<YYINITIAL>"//".*                    {/* Skip one-line comments */}

<YYINITIAL>":"       { return mkToken(COLON); }
<YYINITIAL>"memsize" { return mkToken(MEM_SIZE); }
<YYINITIAL>"pop"     { return mkToken(POP); }
<YYINITIAL>"dup"     { return mkToken(DUP); }
<YYINITIAL>"add"     { return mkToken(ADD); }
<YYINITIAL>"sub"     { return mkToken(SUB); }
<YYINITIAL>"mul"     { return mkToken(MUL); }
<YYINITIAL>"div"     { return mkToken(DIV); }
<YYINITIAL>"rem"     { return mkToken(REM); }
<YYINITIAL>"pow"     { return mkToken(POW); }
<YYINITIAL>"cmpl"    { return mkToken(CMPL) ;}
<YYINITIAL>"cmple"   { return mkToken(CMPLE); }
<YYINITIAL>"cmpe"    { return mkToken(CMPE); }
<YYINITIAL>"printi"  { return mkToken(PRINTI); }
<YYINITIAL>"printb"  { return mkToken(PRINTB); }
<YYINITIAL>"readi"   { return mkToken(READI); }
<YYINITIAL>"readb"   { return mkToken(READB); }
<YYINITIAL>"break"   { return mkToken(BREAK); }
<YYINITIAL>"push"    { return mkToken(PUSH); }
<YYINITIAL>"goto"    { return mkToken(GOTO); }
<YYINITIAL>"store"   { return mkToken(STORE); }
<YYINITIAL>"load"    { return mkToken(LOAD); }
<YYINITIAL>"jz"      { return mkToken(JZ); }
<YYINITIAL>"jnz"     { return mkToken(JNZ); }
<YYINITIAL>"prints"  { return mkToken(PRINTS); }

<YYINITIAL>{digit}+  { return mkToken(INT, Integer.parseInt(yytext())); }

<YYINITIAL>{letter}({letter}|{digit})* { return mkToken(IDENT, yytext()); }

<YYINITIAL>\"        { str = new StringBuffer(); strPos = pos();
                       yybegin(YYSTRING); }
<YYSTRING>\\n        { str.append('\n'); }
<YYSTRING>\\t        { str.append('\t'); }
<YYSTRING>[^\"]*     { str.append(yytext()); }
<YYSTRING>\"         { yybegin(YYINITIAL);
                          return new Token(STRING, strPos, str.toString()); }

. { expressive.Global.instance.error(pos(), "Illegal character: " + yytext()); }
